import arithmetic

def mwt(p,excluded=[]):
  """compute minimum-weight triangulation of a simple polygon p
     by dynamic programming, using integer interval arithmetic.
     The coordinates of p are integers.

     Some edges may be excluded from consideration.
     The polygon p must be oriented counter-clockwise!
     Otherwise, no solution will be found.
     The result is a list of triplets
          (weight, solution, degenerate_flag)
     where weight is given as an interval, sorted by the
     lower bound weight.a of the solution quality
     To show that the result does not depend on assumptions
     about handling point sets which are not in general position,
     degenerate solutions (triangles with three points on a line)
     are considered, but they are flagged as degenerate."""
  opt = {} ## array of optimal (partial) solutions.
  n = len(p)
  for i in range(n-1): # initialization
    o = arithmetic.Interval_integer(0)
             ## calculate the weight only of interior edges
    o.info = None # backtracking information and flag is stored
                  # within the Interval_integer objects
    o.degenerate_flag = False
    opt[i,i+1] = [o]
    # opt[i,j] stores a LIST of all potential candidate solutions.
    
  for l in range(2,n): # l="length" of the edges
    for i in range(n-l):
      j = i+l
      # check for triangulations containing edge (i,j):
      if (i,j) in excluded or (j,i) in excluded:
        ## we are interested in MWT that does not contain those edges
        continue
      if p[i]==p[j]: # this point appears twice on the boundary
        continue
      if (i,j) == (0,n-1):
        edgelength = 0 # final edge is not counted
      else:
        edgelength = arithmetic.length(p[i],p[j])
      for k in range(i+1,j):
        if opt.has_key((i,k)) and opt.has_key((k,j)):
          ori = orientation(p[i],p[k],p[j])
          if ori >= 0:
            in1 = opt[i,k]
            in2 = opt[k,j]
            allcand = [ computesol(in1,in2,n1,n2,k,ori==0,edgelength)
                          for n1 in range(len(in1))
                          for n2 in range(len(in2)) ]
            opt[i,j] = arithmetic.keep_best(opt.get((i,j),[])+allcand)
  # sort final result by lower bounds of intervals:
  allopt = [(x.a,x) for x in opt[0,n-1]]
  allopt.sort()
  opt[0,n-1] = [x[1] for x in allopt]
  # retrace optimal solution:
  return [ (arithmetic.Interval_integer(x.a,x.b),
                       retrace(opt,0,n-1,ind), x.degenerate_flag)
           for x,ind in zip( opt[0,n-1], range(len(opt[0,n-1])) ) ]

def orientation(p1,p2,p3):
  x2 = p2[0]-p1[0]
  y2 = p2[1]-p1[1]
  x3 = p3[0]-p1[0]
  y3 = p3[1]-p1[1]
  return x2*y3-x3*y2

def computesol(l1,l2,i1,i2,k,flag,edgelength):
    r = l1[i1] + l2[i2] + edgelength
    r.info = (k,i1,i2) # k=splitting point.
                       # i1,i2: pointers into sublists
    r.degenerate_flag = (flag or l1[i1].degenerate_flag
                              or l2[i2].degenerate_flag)
    return r

def retrace(opt,i,j,ind):
  """recursively recover the solution, based on the "info"
     fields stored with the solution values in the array "opt".
     Recover the solution for position "ind" in the list "opt".
     The result is a list of triplets (i,k,j), each representing
     the two edges p[i]p[k] and p[k]p[j]."""
  if opt[i,j][ind].info:
    k,i1,i2 = opt[i,j][ind].info
    return [(i,k,j)]+retrace(opt,i,k,i1)+retrace(opt,k,j,i2)
  else:
    return []
